#
# futil.rb
#
#   Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#

require 'ftools'


module FileUtils

  def expand( fn )
    File.expand_path fn
  end

  def mkdir( dname )
    return if File.directory? dname
    Dir.mkdir dname
  end

  def newdir( dname )
    mkdir dname
    dname
  end

  def foreach_f( recv, sim, key, *args )
    if Array === key then
      key.each do |k|
        recv.send sim, k, *args
      end
    else
      recv.send sim, key, *args
    end
  end
  private :foreach_f

  def install( from, to, mode = nil )
    File.install from, to, mode, @verbose
  end

  def cp( from, to )
    foreach_f File, :cp, from, to, @verbose
  end

  def mv( from, to )
    foreach_f File, :mv, from, to, @verbose
  end

  def rm( fn )
    foreach_f File, :unlink, fn
  end

  def rm_f( fn )
    foreach_f File, :rm_f, fn
  end

  def rm_rf( dn )
    begin
      if Array === dn then
        dn.each do |f|
          do_rmrf f if File.directory? f
        end
      else
        do_rmrf dn if File.directory? dn
      end
    rescue
      puts "error in rm_rf"
      $stderr.puts $!
    end
  end

  def do_rmrf( dn )
    foreach_fullpath( dn ) do |fn|
      if File.directory? fn then
        do_rmrf fn
      else
        File.rm_f fn
      end
    end
    Dir.rmdir dn
  end
  private :do_rmrf

  def foreach_fullpath( dn )
    Dir.foreach( dn ) do |fn|
      next if /\A\.\.?\z/o === fn
      yield dn + '/' + fn
    end
  end

  def chmod( mod, f )
    File.chmod mod, *f
  end

end
